searchData={"items":[{"type":"module","title":"Yatm","doc":"The high-level API.","ref":"Yatm.html"},{"type":"function","title":"Yatm.merge/1","doc":"Merges `classes`, provided as a string or a list, returning a string with\nconflicts resolved.\n\nIf a list is provided, anything that isn't a string is discarded. Non-string\nvalues might appear as the result of expressions intended to apply classes\nconditionally.","ref":"Yatm.html#merge/1"},{"type":"function","title":"Examples - Yatm.merge/1","doc":"Conflicting classes that come last override the ones coming before:\n\n    iex> Yatm.merge(\"p-1 p-2\")\n    \"p-2\"\n\nProvide a string or a list of strings:\n\n    iex> Yatm.merge([\"p-1\", \"p-2\"])\n    \"p-2\"\n\nConditionally apply classes:\n\n    iex> Yatm.merge([\"p-1\", true && \"p-2\", false && \"p-3\"])\n    \"p-2\"\n\nAnything that isn't a binary is ignored:\n\n    iex> Yatm.merge([\"p-1\", %{why: \"do this\"}, 42, \"p-2\"])\n    \"p-2\"","ref":"Yatm.html#merge/1-examples"},{"type":"module","title":"Yatm.Parser","doc":"","ref":"Yatm.Parser.html"},{"type":"function","title":"Yatm.Parser.parse/2","doc":"Parses the given `binary` as parse.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the parse (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Yatm.Parser.html#parse/2"},{"type":"function","title":"Options - Yatm.Parser.parse/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Yatm.Parser.html#parse/2-options"},{"type":"module","title":"Yatm.Parser.Helpers","doc":"","ref":"Yatm.Parser.Helpers.html"},{"type":"function","title":"Yatm.Parser.Helpers.arbitrary_value/0","doc":"","ref":"Yatm.Parser.Helpers.html#arbitrary_value/0"},{"type":"function","title":"Yatm.Parser.Helpers.class/3","doc":"","ref":"Yatm.Parser.Helpers.html#class/3"},{"type":"function","title":"Yatm.Parser.Helpers.color/0","doc":"","ref":"Yatm.Parser.Helpers.html#color/0"},{"type":"function","title":"Yatm.Parser.Helpers.custom_property/1","doc":"","ref":"Yatm.Parser.Helpers.html#custom_property/1"},{"type":"function","title":"Yatm.Parser.Helpers.fraction/0","doc":"","ref":"Yatm.Parser.Helpers.html#fraction/0"},{"type":"function","title":"Yatm.Parser.Helpers.identifier/0","doc":"","ref":"Yatm.Parser.Helpers.html#identifier/0"},{"type":"function","title":"Yatm.Parser.Helpers.number/0","doc":"","ref":"Yatm.Parser.Helpers.html#number/0"},{"type":"function","title":"Yatm.Parser.Helpers.percentage/0","doc":"","ref":"Yatm.Parser.Helpers.html#percentage/0"},{"type":"function","title":"Yatm.Parser.Helpers.size_identifier/0","doc":"","ref":"Yatm.Parser.Helpers.html#size_identifier/0"},{"type":"function","title":"Yatm.Parser.Helpers.variant/0","doc":"","ref":"Yatm.Parser.Helpers.html#variant/0"},{"type":"function","title":"Yatm.Parser.Helpers.variants/0","doc":"","ref":"Yatm.Parser.Helpers.html#variants/0"},{"type":"function","title":"Yatm.Parser.Helpers.whitespace/0","doc":"","ref":"Yatm.Parser.Helpers.html#whitespace/0"},{"type":"extras","title":"Yatm","doc":"# Yatm\n\n[![Hex Version](https://img.shields.io/hexpm/v/yatm.svg)](https://hex.pm/packages/yatm)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/yatm)\n\n> #### Warning {: .warning}\n>\n> This project is currently in an experimental proof-of-concept stage. It's\n> still taking shape and many Tailwind features are not yet supported. You're\n> welcome to try it out and encouraged to share your feedback, but don't expect\n> it to serve your production needs just yet.","ref":"readme.html"},{"type":"extras","title":"Usage - Yatm","doc":"Add `yatm` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:yatm, \"~> 0.1.0\"}\n  ]\nend\n```\n\nWherever you potentially need to resolve Tailwind class conflicts, call the\n`merge/1` function like so:\n\n```elixir\ndef button(assigns) do\n  ~H\"\"\"\n   \n    {render_slot(@inner_block)}\n   \n  \"\"\"\nend\n```","ref":"readme.html#usage"},{"type":"extras","title":"Motivation - Yatm","doc":"When using Tailwind with component abstractions, it's not uncommon for the\nauthors of components to provide some basic styling out of the box and for\nusers of those components to want to override some (but not all) of those\nclasses.\n\nWithout a Tailwind class merging utility, component authors usually choose to\neither fully overwrite their out of the box classes or just append the classes\nsupplied by the user to the ones provided in the component. See, for example,\nthe [button\ncomponent](https://github.com/phoenixframework/phoenix/blob/183eb76a88874729cdd5642da03ad7cebd3fa6d3/installer/templates/phx_web/components/core_components.ex#L164)\nincluded in new Phoenix projects:\n\n```elixir\ndef button(assigns) do\n  ~H\"\"\"\n   \n    {render_slot(@inner_block)}\n   \n  \"\"\"\nend\n```\n\nIf the user provides some additional classes with the `@class` attribute, these\nwill get appended to the existing ones. This can very easily lead to [class\nconflicts](https://tailwindcss.com/docs/styling-with-utility-classes#conflicting-utility-classes).\n\nFor example, if the user were to supply a `text-xl` class, there's no guarantee\n(in the general case) that Tailwind will build a CSS file where the `text-xl`\nis declared after the `text-sm` class (i.e. overriding the `font-size` property\nwith the CSS cascade). If it did, this might still be a problem if a `text-sm`\nwere intended to override a `text-xl` somewhere else in the codebase.\n\nThis is the problem that a Tailwind class merging utility is designed to\naddress. It works by parsing the list of concatenated classes and using\nknowledge of the CSS properties targeted by each Tailwind utility class to\nremove classes that are meant to be overriden by conflicting classes appearing\nlater in the string.\n\nCalling `Yatm.merge(\"text-sm text-xl\")` will return `\"text-xl\"` and calling\n`Yatm.merge(\"text-xl text-sm\")` will return `\"text-sm\"`.","ref":"readme.html#motivation"},{"type":"extras","title":"Roadmap - Yatm","doc":"","ref":"readme.html#roadmap"},{"type":"extras","title":"Finish the `merge` utility - Yatm","doc":"The first milestone is to finish building a `merge/1` function that can take\nthe same inputs as the regular `class` attribute in HEEx templates–a binary or\na list of binaries with nil and false values being discarded.","ref":"readme.html#finish-the-merge-utility"},{"type":"extras","title":"Concerns about (re)compilation - Yatm","doc":"I'm currently implementing this using\n[nimble_parsec](https://hex.pm/packages/nimble_parsec). I'm concerned that\ncompiling the parser is taking on the order of dozens of seconds and the\nresulting BEAM file is in the MBs. If this compilation only needs to run once,\nthat could be fine.\n\nThe problem is that the implementation will eventually have to take into\naccount the user's custom Tailwind theme, which could add new utility classes.\nIf the theme changes, then maybe that results in a parser recompilation, which\nis not ideal.\n\nI still need to explore the space of solutions and problems that might arise as\nthe implementation develops.","ref":"readme.html#concerns-about-re-compilation"},{"type":"extras","title":"Concerns about runtime performance - Yatm","doc":"My knowledge of the Phoenix rendering pipeline has some gaps but, as far as I\ncan tell, the code above would have to run the merge on every template\nre-render.\n\nMaybe the run time of the function is so small that there's not a big reason to\ntry to optimize it away but I'm not sure at this moment, I'd like to do some\nbenchmarking at some point.\n\nOther libraries memoize these computations. For example,\n[tw_merge](https://hex.pm/packages/tw_merge), which was forked from\n[turboprop](https://hex.pm/packages/turboprop), requires adding a cache process\nto your supervision tree.\n\nTurboprop's merge utility seems to have been implemented as a port of\n[tailwind-merge](https://github.com/dcastil/tailwind-merge), the canonical\nTailwind merge utility in the JavaScript ecosystem, which itself implements an\n[LRU\ncache](https://github.com/dcastil/tailwind-merge/blob/f4eacb6bc1800031147a153fcf20e586b277320e/src/lib/lru-cache.ts).\n\nIt could be that I'm just too early in my Elixir journey, but the idea of\nadding a process to the supervision tree just to merge strings together doesn't\nfeel super enticing. Could we find a better solution? Read on!","ref":"readme.html#concerns-about-runtime-performance"},{"type":"extras","title":"Building a custom EEx engine - Yatm","doc":"What if the templating engine was able to do this Tailwind class merging\nitself? No need to manually call the `merge` function every time. EEx provides\nthe [Engine behaviour](https://hexdocs.pm/eex/EEx.Engine.html) to enable this\nkind of extension. As far as I understand, EEx templates are usually used in a\nway in which they're precompiled, so maybe there's a way to \"memoize\" these\n(maybe expensive) class merging operations within the templates themselves.\n\nIf a component's class attribute takes a list of static string values known at\ncompile-time, we could run the merge and just inline the result into the\ncompiled template code, resulting in zero run-time cost.\n\nIf there's some run-time logic that makes the class value dynamic, it might\nstill be possible to transform the code and precompute the necessary merge\noperations at compile-time. For example, the code `class={[\"p-1 text-sm\", if\nsome_condition, do: \"text-xl\", else: \"text-2xl\"]}` could be transformed into\nsomething like `class={if some_condition, do: \"p-1 text-xl\", else: \"p-1\ntext-2xl\"}` so the minimum amount of work is left to be done at run-time.\n\nWe get the best of both worlds: no run-time cost (not even for cache misses)\nand no extra cache process and the latency, jitter and memory footprint that\ncould potentially have. The API is also better as merging happens without\nhaving to call the merge utility explicitly every time, the template engine\njust takes care of it according to a config value. The cost would be larger\ncompile times, obviously.","ref":"readme.html#building-a-custom-eex-engine"},{"type":"extras","title":"Integrating into Phoenix? - Yatm","doc":"Wouldn't it be great if Phoenix shipped with this functionality out of the box?\nI think so. Given that Tailwind is so commonly used, it'd be nice to have\nTailwind class merging as one of the features of the templating engine shipped\nwith Phoenix. I assume there'd be an easy way to switch this feature on and off\nso that projects not using Tailwind wouldn't have to pay the compilation price.","ref":"readme.html#integrating-into-phoenix"},{"type":"extras","title":"Ecosystem - Yatm","doc":"A list of alternatives and other Tailwind-related packages:\n\n- [tailwind](https://hex.pm/packages/tailwind).\n- [tailwind_merge](https://hex.pm/packages/tailwind_merge).\n- [twix](https://hex.pm/packages/twix).\n- [tw_merge](https://hex.pm/packages/tw_merge).\n- [tailwind_formatter](https://hex.pm/packages/tailwind_formatter).\n- [tails](https://hex.pm/packages/tails).\n- [turboprop](https://hex.pm/packages/turboprop).","ref":"readme.html#ecosystem"},{"type":"extras","title":"License - Yatm","doc":"[Blue Oak Model License 1.0.0](LICENSE.md), a [modern\nalternative](https://writing.kemitchell.com/2019/03/09/Deprecation-Notice.html)\nto the MIT license.","ref":"readme.html#license"},{"type":"extras","title":"License","doc":"# Blue Oak Model License\n\nVersion 1.0.0","ref":"license.html"},{"type":"extras","title":"Purpose - License","doc":"This license gives everyone as much permission to work with\nthis software as possible, while protecting contributors\nfrom liability.","ref":"license.html#purpose"},{"type":"extras","title":"Acceptance - License","doc":"In order to receive this license, you must agree to its\nrules.  The rules of this license are both obligations\nunder that agreement and conditions to your license.\nYou must not do anything with this software that triggers\na rule that you cannot or will not follow.","ref":"license.html#acceptance"},{"type":"extras","title":"Copyright - License","doc":"Each contributor licenses you to do everything with this\nsoftware that would otherwise infringe that contributor's\ncopyright in it.","ref":"license.html#copyright"},{"type":"extras","title":"Notices - License","doc":"You must ensure that everyone who gets a copy of\nany part of this software from you, with or without\nchanges, also gets the text of this license or a link to\n .","ref":"license.html#notices"},{"type":"extras","title":"Excuse - License","doc":"If anyone notifies you in writing that you have not\ncomplied with [Notices](#notices), you can keep your\nlicense by taking all practical steps to comply within 30\ndays after the notice.  If you do not do so, your license\nends immediately.","ref":"license.html#excuse"},{"type":"extras","title":"Patent - License","doc":"Each contributor licenses you to do everything with this\nsoftware that would otherwise infringe any patent claims\nthey can license or become able to license.","ref":"license.html#patent"},{"type":"extras","title":"Reliability - License","doc":"No contributor can revoke this license.","ref":"license.html#reliability"},{"type":"extras","title":"No Liability - License","doc":"***As far as the law allows, this software comes as is,\nwithout any warranty or condition, and no contributor\nwill be liable to anyone for any damages related to this\nsoftware or this license, under any kind of legal claim.***","ref":"license.html#no-liability"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}